@@ -120,7 +120,7 @@ class Runner(object):
         return 10
 
 
-class Shel(object):
+class Shell(object):
     def __init__(self, servurl, appid, appkey, mqtturi, gateway, device, runner, name=None, interval=30):
         try:
             import urlparse as parse
@@ -280,40 +280,53 @@ class CommandRunner(Runner):
         needret = cmdd.get('return', False)
 
         def run():
+            import time, socket
             print(action, args)
             dtu.sendEvent(dev_id, 'warning', 'start %s' % action)
+            outs = []
+            mid = socket.gethostname()
+
+            def notify(finish=False):
+                if needret:
+                    return
+                text = ''.join(outs)
+                dtu.sendNotify(dev_id, {'type': 'result', 'text': text, 'value': value, 'finish': finish, 'sid': sid, 'mid': mid})
+                del outs[:]
+
             try:
+                env = value.get('env') if isinstance(value, dict) else None
+                if env:
+                    import os
+                    env.update(os.environ)
                 # subprocess.call()
-                p = subprocess.Popen(args, shell=cmdd.get('shell'), stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=os.getcwd(), bufsize=1)
+                p = subprocess.Popen(args, shell=cmdd.get('shell'), stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=os.getcwd(), bufsize=1, env=env)
                 pipe = Pipe(p.stdout)
                 last = time.time()
-                outs = []
-
-                def notify(finish=False):
-                    if needret:
-                        return
-                    text = ''.join(outs)
-                    dtu.sendNotify(dev_id, {'type': 'result', 'text': text, 'value': value, 'finish': finish, 'sid': sid})
-                    # print(text)
-                    del outs[:]
-
+                start = time.time()
+                outs.append('\n>run start...\n\n')
+                notify()
                 while self.running:
-                    notify()
+                    # notify()
                     try:
                         out = pipe.readline(1)
-                        if not out:
+                        if out:
+                            outs.append(out.decode('utf-8'))
+                        elif p.poll() != None:
                             break
-                        outs.append(out.decode('utf-8'))
                     except ReadTimeoutException as e:
                         # import traceback
                         # traceback.print_exc()
                         pass
                     timeoff = (time.time() - last)
-                    if timeoff > 2 and outs or timeoff > 5:
+                    if timeoff > 1 and outs or timeoff > 5:
                         last = time.time()
                         notify()
+                # print("self.running out", self.running, p.returncode)
+                code = p.returncode
+                ctime = int((time.time() - start) * 1000)
+                outs.append('\n>run end, exit code=%s, time %dms!\n' % (code, ctime))
                 notify(True)
-
+                print('%s exit code=%s, time %dms!' % (action, code, ctime))
                 try:
                     pipe.close()
                 except:
@@ -326,6 +339,13 @@ class CommandRunner(Runner):
                 import traceback
                 traceback.print_exc()
                 dtu.sendEvent(dev_id, 'error', str(e))
+                try:
+                    outs.append('\nerror!!!\n')
+                    outs.append(str(e))
+                    outs.append('\nrun end, time %dms!\n' % (int((time.time() - start) * 1000)))
+                    notify(True)
+                except:
+                    pass
 
         self.start_thread(run)
 
@@ -456,7 +476,7 @@ if __name__ == '__main__':
             runner_args = {'commands': args.command}
         # print(runner_clz, runner_args)
         runner = runner_clz(**runner_args)
-        sh = Shel(servurl=args.server, appid=args.appid, appkey=args.appkey, mqtturi=args.mqtt, gateway=args.gateway, device=args.device, runner=runner, name=args.name, interval=int(args.interval))
+        sh = Shell(servurl=args.server, appid=args.appid, appkey=args.appkey, mqtturi=args.mqtt, gateway=args.gateway, device=args.device, runner=runner, name=args.name, interval=int(args.interval))
         sh.start()
         while sh.running:
             time.sleep(1)
